/*
 * PKCS #11 PAM Login Module
 * Copyright (C) 2003 Mario Strasser <mast@gmx.net>,
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * $Id$
 */

#ifndef SMARTCARD_H
#define SMARTCARD_H

#include "cert_st.h"

/* forward opaque declarations */
typedef struct cert_object_str cert_object_t;
typedef struct pkcs11_handle_str pkcs11_handle_t;

int crypto_init(char *arg);

int load_pkcs11_module(char *module, pkcs11_handle_t **h);

int init_pkcs11_module(pkcs11_handle_t *h);

void release_pkcs11_module(pkcs11_handle_t *h);

int find_slot_by_number(pkcs11_handle_t *h, int slot_num);
int find_slot_by_number_and_label(pkcs11_handle_t *h, 
				  int slot_num,
				  const char *slot_label);

int open_pkcs11_session(pkcs11_handle_t *h, unsigned int slot);

const char *get_slot_label(pkcs11_handle_t *h);

int wait_for_token(pkcs11_handle_t *h, 
                   int wanted_slot_num,
                   const char *wanted_slot_label);

int pkcs11_login(pkcs11_handle_t *h, char *password);

int close_pkcs11_session(pkcs11_handle_t *h);

int get_user_certificates(pkcs11_handle_t *h, cert_object_t ***certs, int *count);
const X509 *get_X509_certificate(cert_object_t *cert);
int free_certificates(pkcs11_handle_t *h, cert_object_t **certs, int count);

int sign_value(pkcs11_handle_t *h, cert_object_t *cert, unsigned char *data,
		unsigned long length, unsigned char **signature, 
		unsigned long*signature_length);

int get_random_value(unsigned char *data, int length);

#endif /* PKCS11_H */
